import { RequestType, NotificationType } from 'vscode-jsonrpc';
/**
 * Defines the capabilities provided by the client.
 */
export interface ClientCapabilities {
}
/**
 * Defines how the host (editor) should sync
 * document changes to the language server.
 */
export declare enum TextDocumentSyncKind {
    /**
     * Documents should not be synced at all.
     */
    None = 0,
    /**
     * Documents are synced by always sending the full content
     * of the document.
     */
    Full = 1,
    /**
     * Documents are synced by sending the full content on open.
     * After that only incremental updates to the document are
     * send.
     */
    Incremental = 2,
}
/**
 * Completion options
 */
export interface CompletionOptions {
    /**
     * The server provides support to resolve additional
     * information for a completion item.
     */
    resolveProvider?: boolean;
    /**
     * The characters that trigger completion automatically.
     */
    triggerCharacters?: string[];
}
/**
 * Signature help options
 */
export interface SignatureHelpOptions {
    /**
     * The characters that trigger signature help
     * automatically.
     */
    triggerCharacters?: string[];
}
/**
 * Defines the capabilities provided by the language
 * server
 */
export interface ServerCapabilities {
    /** Defines how text documents are synced. */
    textDocumentSync?: number;
    /** The server provides hover support. */
    hoverProvider?: boolean;
    /** The server provides completion support. */
    completionProvider?: CompletionOptions;
    /** The server provides signature help support. */
    signatureHelpProvider?: SignatureHelpOptions;
    /** The server provides goto definition support. */
    definitionProvider?: boolean;
    /** The server provides find references support. */
    referencesProvider?: boolean;
    /** The server provides document highlight support. */
    documentHighlightProvider?: boolean;
}
/**
 * The initialize method is sent from the client to the server.
 * It is send once as the first method after starting up the
 * worker.
 */
export declare namespace InitializeRequest {
    const type: RequestType<InitializeParams, InitializeResult, InitializeError>;
}
/**
 * The initialize parameters
 */
export interface InitializeParams {
    /**
     * The rootPath of the workspace. Is null
     * if no folder is open.
     */
    rootPath: string;
    /**
     * The capabilities provided by the client (editor)
     */
    capabilities: ClientCapabilities;
}
/**
 * The result returned from an initilize request.
 */
export interface InitializeResult {
    /**
     * The capabilities the language server provides.
     */
    capabilities: ServerCapabilities;
}
/**
 * The error returned if the initilize request fails.
 */
export interface InitializeError {
    /**
     * Indicates whether the client should retry to send the
     * initilize request after showing the message provided
     * in the {@link ResponseError}
     */
    retry: boolean;
}
/**
 * A shutdown request is sent from the client to the server.
 * It is send once when the client descides to shutdown the
 * server. The only event that is sent after a shudown request
 * is the exit event.
 */
export declare namespace ShutdownRequest {
    const type: RequestType<void, void, void>;
}
/**
 * The exit event is sent from the client to the server to
 * ask the server to exit its process.
 */
export declare namespace ExitNotification {
    const type: NotificationType<void>;
}
/**
 * The configuration change notification is sent from the client to the server
 * when the client's configuration has changed. The notification contains
 * the changed configuration as defined by the language client.
 */
export declare namespace DidChangeConfigurationNotification {
    const type: NotificationType<DidChangeConfigurationParams>;
}
/**
 * The parameters of a change configuration notification.
 */
export interface DidChangeConfigurationParams {
    /**
     * The actual changed settings
     */
    settings: any;
}
/**
 * The message type
 */
export declare enum MessageType {
    /** An error message. */
    Error = 1,
    /** A warning message. */
    Warning = 2,
    /** An information message. */
    Info = 3,
    /** A log message. */
    Log = 4,
}
/**
 * The show message notification is sent from a server to a client to ask
 * the client to display a particular message in the user interface.
 */
export declare namespace ShowMessageNotification {
    const type: NotificationType<ShowMessageParams>;
}
/**
 * The parameters of a notification message.
 */
export interface ShowMessageParams {
    /**
     * The message type. See {@link MessageType}
     */
    type: number;
    /**
     * The actual message
     */
    message: string;
}
/**
 * The log message notification is send from the server to the client to ask
 * the client to log a particular message.
 */
export declare namespace LogMessageNotification {
    let type: NotificationType<LogMessageParams>;
}
/**
 * The log message parameters.
 */
export interface LogMessageParams {
    /**
     * The message type. See {@link MessageType}
     */
    type: number;
    /**
     * The actual message
     */
    message: string;
}
/**
 * Position in a text document expressed as zero-based line and character offset.
 */
export interface Position {
    /**
     * Line position in a document (zero-based).
     */
    line: number;
    /**
     * Character offset on a line in a document (zero-based).
     */
    character: number;
}
/**
 * A range in a text document expressed as (zero-based) start and end positions.
 */
export interface Range {
    /**
     * The range's start position
     */
    start: Position;
    /**
     * The range's end position
     */
    end: Position;
}
/**
 * Represents a location inside a resource, such as a line
 * inside a text file.
 */
export declare class Location {
    uri: string;
    range: Range;
}
/**
 * A literal to identify a text document in the client.
 */
export interface TextDocumentIdentifier {
    /**
     * The text document's uri.
     */
    uri: string;
}
/**
 * A literal to define the position in a text document.
 */
export interface TextDocumentPosition extends TextDocumentIdentifier {
    /**
     * The position inside the text document.
     */
    position: Position;
}
/**
 * The document open notification is sent from the client to the server to signal
 * newly opened text documents. The document's truth is now managed by the client
 * and the server must not try to read the document's truth using the document's
 * uri.
 */
export declare namespace DidOpenTextDocumentNotification {
    const type: NotificationType<DidOpenTextDocumentParams>;
}
/**
 * The parameters send in a open text document notification
 */
export interface DidOpenTextDocumentParams extends TextDocumentIdentifier {
    /**
     * The content of the opened  text document.
     */
    text: string;
}
/**
 * The document change notification is sent from the client to the server to signal
 * changes to a text document. The notification's parameters are either a single
 * {@link DidChangeTextDocumentParams} or an array of {@link DidChangeTextDocumentParams}.
 */
export declare namespace DidChangeTextDocumentNotification {
    const type: NotificationType<DidChangeTextDocumentParams | DidChangeTextDocumentParams[]>;
}
/**
 * The arguments describing a change to a text document. If range and rangeLength are omitted
 * the new text is considered to be the full content of the document.
 */
export interface DidChangeTextDocumentParams extends TextDocumentIdentifier {
    /**
     * The range of the document that changed.
     */
    range?: Range;
    /**
     * The length of the range that got replaced.
     */
    rangeLength?: number;
    /**
     * The new text of the document.
     */
    text: string;
}
/**
 * The document close notification is sent from the client to the server when
 * the document got closed in the client. The document's truth now exists
 * where the document's uri points to (e.g. if the document's uri is a file uri
 * the truth now exists on disk).
 */
export declare namespace DidCloseTextDocumentNotification {
    const type: NotificationType<TextDocumentIdentifier>;
}
/**
 * The watched files notification is sent from the client to the server when
 * the client detects changes to file watched by the lanaguage client.
 */
export declare namespace DidChangeWatchedFilesNotification {
    const type: NotificationType<DidChangeWatchedFilesParams>;
}
/**
 * The watched files change notification's parameters.
 */
export interface DidChangeWatchedFilesParams {
    /**
     * The actual file events.
     */
    changes: FileEvent[];
}
/**
 * The file event type
 */
export declare enum FileChangeType {
    /** The file got created. */
    Created = 1,
    /** The file got changed. */
    Changed = 2,
    /** The file got deleted. */
    Deleted = 3,
}
/**
 * An event describing a file change.
 */
export interface FileEvent {
    /** The file's uri. */
    uri: string;
    /** teh change type. */
    type: number;
}
/**
 * Diagnostics notification are sent from the server to the client to signal
 * results of validation runs.
 */
export declare namespace PublishDiagnosticsNotification {
    const type: NotificationType<PublishDiagnosticsParams>;
}
/**
 * The publish diagnostic notification's parameters.
 */
export interface PublishDiagnosticsParams {
    /**
     * The URI for which diagnostic information is reported.
     */
    uri: string;
    /**
     * An array of diagnostic information items.
     */
    diagnostics: Diagnostic[];
}
/**
 * The diagnostic's serverity.
 */
export declare enum DiagnosticSeverity {
    /** Reports an error. */
    Error = 1,
    /** Reports a warning. */
    Warning = 2,
    /** Reports an information. */
    Information = 3,
    /** Reports a hint. */
    Hint = 4,
}
/**
 * A diagnostic item found in a diagnostic notification.
 */
export interface Diagnostic {
    /**
     * The range at which the message applies
     */
    range: Range;
    /**
     * The diagnostic's severity. Can be omitted. If omitted it is up to the
     * client to interpret diagnostics as error, warning, info or hint.
     */
    severity?: number;
    /**
     * The diagnostic's code. Can be omitted.
     */
    code?: number | string;
    /**
     * The diagnostic message.
     */
    message: string;
}
export declare enum CompletionItemKind {
    Text = 1,
    Method = 2,
    Function = 3,
    Constructor = 4,
    Field = 5,
    Variable = 6,
    Class = 7,
    Interface = 8,
    Module = 9,
    Property = 10,
    Unit = 11,
    Value = 12,
    Enum = 13,
    Keyword = 14,
    Snippet = 15,
    Color = 16,
    File = 17,
    Reference = 18,
}
export interface CompletionItem {
    label: string;
    kind?: number;
    detail?: string;
    documentation?: string;
    sortText?: string;
    filterText?: string;
    insertText?: string;
    textEdit?: TextEdit;
}
export declare namespace CompletionItem {
    function create(label: string): CompletionItem;
}
export interface TextEdit {
    range: Range;
    newText: string;
}
export declare namespace TextEdit {
    function replace(range: Range, newText: string): TextEdit;
    function insert(position: Position, newText: string): TextEdit;
    function del(range: Range): TextEdit;
}
export declare namespace CompletionRequest {
    const type: RequestType<TextDocumentPosition, CompletionItem[], void>;
}
export declare namespace CompletionResolveRequest {
    const type: RequestType<CompletionItem, CompletionItem, void>;
}
export declare type MarkedString = string | {
    language: string;
    value: string;
};
/**
 * The result of a hove request.
 */
export interface Hover {
    /**
     * The hover's content
     */
    contents: MarkedString | MarkedString[];
    /**
     * An optional range
     */
    range?: Range;
}
export declare namespace HoverRequest {
    const type: RequestType<TextDocumentPosition, Hover, void>;
}
export interface ParameterInformation {
    label: string;
    documentation?: string;
}
export declare namespace ParameterInformation {
    function create(label: string, documentation?: string): ParameterInformation;
}
export interface SignatureInformation {
    label: string;
    documentation?: string;
    parameters?: ParameterInformation[];
}
export declare namespace SignatureInformation {
    function create(label: string, documentation?: string): SignatureInformation;
}
export interface SignatureHelp {
    signatures?: SignatureInformation[];
    activeSignature?: number;
    activeParameter?: number;
}
export declare namespace SignatureHelpRequest {
    const type: RequestType<TextDocumentPosition, SignatureHelp, void>;
}
/**
 * The definition of a symbol is one or many [locations](#Location)
 */
export declare type Definition = Location | Location[];
export declare namespace DefinitionRequest {
    const type: RequestType<TextDocumentPosition, Definition, void>;
}
export interface ReferenceParams extends TextDocumentPosition {
    options: {
        includeDeclaration: boolean;
    };
}
export declare namespace ReferencesRequest {
    const type: RequestType<ReferenceParams, Location[], void>;
}
export declare enum DocumentHighlightKind {
    Text = 1,
    Read = 2,
    Write = 3,
}
export interface DocumentHighlight {
    range: Range;
    kind?: number;
}
export declare namespace DocumentHighlightRequest {
    const type: RequestType<TextDocumentPosition, DocumentHighlight[], void>;
}
